"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Kernel = void 0;
const awilix_1 = require("awilix");
const awilix_koa_1 = require("awilix-koa");
const koa_1 = __importDefault(require("koa"));
const koa_qs_1 = __importDefault(require("koa-qs"));
const middleware_1 = require("./middleware");
const node_util_1 = require("node:util");
const awilix_manager_1 = require("awilix-manager");
class Kernel {
    constructor(options) {
        this.options = options;
        this.logger = this.createLogger();
        this.container = this.createContainer();
        this.app = (0, koa_qs_1.default)(this.createApp());
    }
    async boot() {
        await this.buildContainer();
        await (0, awilix_manager_1.asyncInit)(this.container);
        this.addMiddlewares();
        this.loadControllers();
    }
    async serve() {
        let host = this.options.get('host');
        let port = this.options.get('port');
        this.server = this.app.listen(port, host);
        await new Promise((Ok) => this.server.on('listening', Ok));
        this.logger.info('Server running', { host, port, url: `http://${host}:${port}` });
    }
    async shutdown() {
        await (0, awilix_manager_1.asyncDispose)(this.container);
        if (true === Boolean(this.server)) {
            await (0, node_util_1.promisify)(this.server.close).call(this.server);
        }
    }
    createContainer() {
        return (0, awilix_1.createContainer)({ injectionMode: 'CLASSIC', strict: true });
    }
    async buildContainer() {
        let builder = this.createContainerBuilder();
        await builder.build(this.container);
    }
    addMiddlewares() {
        this.app.use((0, middleware_1.errorHandler)(this.logger));
        this.app.use((0, middleware_1.containerScope)(this.container));
        for (let middleware of this.middlewares()) {
            this.app.use(middleware);
        }
    }
    createApp() {
        return new koa_1.default();
    }
    getControllerPaths() {
        return 'controllers/*.js';
    }
    middlewares() {
        return [];
    }
    loadControllers() {
        let controllers = (0, awilix_koa_1.loadControllers)(this.getControllerPaths(), { cwd: this.getCWD() });
        this.app.use(controllers);
    }
}
exports.Kernel = Kernel;
//# sourceMappingURL=kernel.js.map